# .
# chatviewer.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/gui/tagless_viewer'
require 'lib/gui/hex_viewer'

module Watobo
  module Gui
    
    SEL_TYPE_GREP = 0
    SEL_TYPE_HIGHLIGHT = 1
    
     class TextViewer < FXVerticalFrame
      
      include Watobo::Gui::Utils
      attr_accessor :max_len
      
      def style=(style)
        @textviewer.style = style
      end
      
      def editable=(value)
        @textviewer.editable = value
      end
      
      def onHighlight(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        @textviewer.highlight(@textFilter.value)
        @textviewer.makeMatchVisible(0)
        @last_filter_action = SEL_TYPE_HIGHLIGHT
      end
      
      def highlight(pattern)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        @textviewer.highlight(pattern)
        @textviewer.makeMatchVisible(0)
      end
      
      def onGrep(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        pattern = @textFilter.value
        @textviewer.filter(pattern)
        @textviewer.highlight(pattern)
        @textviewer.makeMatchVisible(0)
        @last_filter_action = SEL_TYPE_GREP
      end
      
      def onReset(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
      end
      
      def setText(text)
        @text = text
        @textviewer.max_len = @max_len
      @textviewer.setText(text)
      end
      
      def getText
        @textviewer.textbox.to_s
      end
      
      def initialize(owner, opts)
        super(owner, opts)
        @textFilter = FXDataTarget.new("")
        @text_matches = []
        
        @textbox = nil
        @tabbook = nil
        @viewers = []
        @style = 2 # default style
        @text = ''
        @max_len = 5000
         @last_filter_action = SEL_TYPE_HIGHLIGHT
        
        text_view_header = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM, :padding => 0)
        FXTextField.new(text_view_header, 10, @textFilter, FXDataTarget::ID_VALUE, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X)
        
        grep_button = FXButton.new(text_view_header, "Grep", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        grep_button.connect(SEL_COMMAND, method(:onGrep))
        
        highlight_button = FXButton.new(text_view_header, "Highlight", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        highlight_button.connect(SEL_COMMAND, method(:onHighlight))
        
        reset_button = FXButton.new(text_view_header, "Reset", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        reset_button.connect(SEL_COMMAND, method(:onReset))
        
        text_box_frame = FXVerticalFrame.new(self, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding => 0)
        
        @textviewer = SimpleTextView.new(text_box_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y,:padding => 0)
        @textviewer.style = 1
        @textviewer.editable = false
        @textviewer.textbox.textStyle -= TEXT_WORDWRAP
        
        @textFilter.connect(SEL_COMMAND) do |sender, sel, event|
          
          if @last_filter_action == SEL_TYPE_HIGHLIGHT then 
           onHighlight(sender, sel, event)
          elsif @last_filter_action == SEL_TYPE_GREP then
            onGrep(sender, sel, event)
          end
          
        end
      end
      
    end
    
    class RequestViewer < FXVerticalFrame
      
      include Watobo::Gui::Utils
      attr_accessor :max_len
      
      def setText(text)
        @text = text
        @textviewer.max_len = @max_len
        index = @tabBook.current
        @viewers[index].setText(text)
      end
      
      def getText
        index = @tabBook.current
        @viewers[index].getText()
      end
      
      def highlight(pattern)
        begin
         index = @tabBook.current
        @viewers[index].highlight(pattern)
      rescue
        end
      end
      
      def initialize(owner, opts)
        super(owner, opts)
        @tabbook = nil
        @viewers = []
        @text = ''
        @max_len = 5000
        
      
        @tabBook = FXTabBook.new(self, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
        @tabBook.connect(SEL_COMMAND) {
          begin
            getApp().beginWaitCursor() 
            setText(@text)
          ensure
            getApp().endWaitCursor()
          end
        }
        textviewer_tab = FXTabItem.new(@tabBook, "Text", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        @textviewer = TextViewer.new(tab_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y,:padding => 0)
         @textviewer.style = 1
        @textviewer.editable = false
       
        @viewers.push @textviewer
       
        FXTabItem.new(@tabBook, "Hex", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        @hexViewer = HexViewer.new(tab_frame)
        @viewers.push @hexViewer
      end
      
    end
    
    
    class ResponseViewer < FXVerticalFrame
      
      include Watobo::Gui::Utils
      attr_accessor :max_len
      
      def setText(text)
        @text = text
        @textviewer.max_len = @max_len
        index = @tabBook.current
        @viewers[index].setText(text)
      end
      
      def getText
        index = @tabBook.current
        @viewers[index].getText()
      end
      
      def highlight(pattern)
        begin
         index = @tabBook.current
        @viewers[index].highlight(pattern)
      rescue
        end
      end
      
      def initialize(owner, opts)
        super(owner, opts)
       
        @tabbook = nil
        @viewers = []
        @text = ''
        @max_len = 5000
        
      
        @tabBook = FXTabBook.new(self, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
        @tabBook.connect(SEL_COMMAND) {
          begin
            getApp().beginWaitCursor() 
            setText(@text)
          ensure
            getApp().endWaitCursor()
          end
        }
        textviewer_tab = FXTabItem.new(@tabBook, "Text", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        @textviewer = TextViewer.new(tab_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y,:padding => 0)
       @textviewer.style = 2
        @textviewer.editable = false
        @viewers.push @textviewer
       
       taglessviewer_tab = FXTabItem.new(@tabBook, "Tagless", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        tagless_frame = FXVerticalFrame.new(tab_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
        #text_view_header = FXHorizontalFrame.new(tagless_frame, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM)
        @taglessviewer = TaglessViewer.new(tagless_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        @viewers.push @taglessviewer
       
        FXTabItem.new(@tabBook, "Hex", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        @hexViewer = HexViewer.new(tab_frame)
        @viewers.push @hexViewer
      end
      
    end
    
    class ChatViewer < FXVerticalFrame
      
      include Watobo::Gui::Utils
      attr_accessor :max_len
      
      def highlight_style=(style)
        @textviewer.style = style
      end
      
      def onHighlight(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        @textviewer.highlight(@textFilter.value)
        @textviewer.makeMatchVisible(0)
        @last_button_pressed = SEL_TYPE_HIGHLIGHT
      end
      
      def highlight(pattern)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        @textviewer.highlight(pattern)
        @textviewer.makeMatchVisible(0)
      end
      
      def onGrep(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
        pattern = @textFilter.value
        @textviewer.filter(pattern)
        @textviewer.highlight(pattern)
        @textviewer.makeMatchVisible(0)
        @last_button_pressed = SEL_TYPE_GREP
      end
      
      def onReset(sender, sel, event)
        @textviewer.max_len = 0
        @textviewer.setText(@text)
      end
      
      def setText(text)
        @text = text
        @textviewer.max_len = @max_len
        #@taglessviewer.max_len = @max_len
        index = @tabBook.current
        @viewers[index].setText(text)
      end
      
      def getText
        @textviewer.textbox.to_s
      end
      
      def initialize(owner, opts)
        super(owner, opts)
        @textFilter = FXDataTarget.new("")
        @text_matches = []
        
        @textbox = nil
        @tabbook = nil
        @viewers = []
        @style = 2 # default style
        @text = ''
        @max_len = 5000
        
        @last_button_pressed = SEL_TYPE_HIGHLIGHT
        
        #
        @tabBook = FXTabBook.new(self, nil, 0, LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT)
        @tabBook.connect(SEL_COMMAND) {
          begin
            getApp().beginWaitCursor() 
            setText(@text)
          ensure
            getApp().endWaitCursor()
          end
        }
        textviewer_tab = FXTabItem.new(@tabBook, "Text", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        
        text_view_header = FXHorizontalFrame.new(tab_frame, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM, :padding => 0)
        FXTextField.new(text_view_header, 10, @textFilter, FXDataTarget::ID_VALUE, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X)
        
        grep_button = FXButton.new(text_view_header, "Grep", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        grep_button.connect(SEL_COMMAND, method(:onGrep))
        
        highlight_button = FXButton.new(text_view_header, "Highlight", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        highlight_button.connect(SEL_COMMAND, method(:onHighlight))
        
        reset_button = FXButton.new(text_view_header, "Reset", nil, nil, 0, FRAME_RAISED|FRAME_THICK)
        reset_button.connect(SEL_COMMAND, method(:onReset))
        
        text_box_frame = FXVerticalFrame.new(tab_frame, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding => 0)
        
        @textviewer = SimpleTextView.new(text_box_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y,:padding => 0)
        @textviewer.style = 1
        @textviewer.editable = false
        
        @textFilter.connect(SEL_COMMAND) do |sender, sel, event|          
          if @last_button_pressed == SEL_TYPE_HIGHLIGHT then 
            onHighlight(sender, sel, event)
          elsif @last_button_pressed == SEL_TYPE_GREP then
            onGrep(sender, sel, event)
          end          
        end
        
        @viewers.push @textviewer
       
        taglessviewer_tab = FXTabItem.new(@tabBook, "Tagless", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        tagless_frame = FXVerticalFrame.new(tab_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK, :padding=>0)
        #text_view_header = FXHorizontalFrame.new(tagless_frame, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM)
        @taglessviewer = TaglessViewer.new(tagless_frame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y, :padding => 0)
        @taglessviewer.style = 1
        @taglessviewer.editable = false
        @viewers.push @taglessviewer
        
        FXTabItem.new(@tabBook, "Hex", nil)
        tab_frame = FXVerticalFrame.new(@tabBook, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED)
        @hexViewer = HexViewer.new(tab_frame)
        @viewers.push @hexViewer
      end
      
    end
    ###################### end of namespace ############################
  end
end