# .
# active_check.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'digest/md5'
module Watobo
  module Core
    
    class ActiveCheck  < Watobo::Core::Session    # Base Class for Passive Checks
      include Watobo::Core
      include Watobo::Utils
      
      attr :info
      attr :numChecks
      
      #   @@running_checks = 0
      @@max_checks = 5
      
      @@pool = []
      @@pool_mutex = Mutex.new
      @@pool_cv = ConditionVariable.new
      
      @@login_in_progress = false
      
      @@status = :running # :running, :paused, :canceled
      
      
      
      def addFinding(request, response, details)
        @@lock.synchronize {
          
          t = Time.now
          now = t.strftime("%m/%d/%Y@%H:%M:%S")  
          
          new_details = Hash.new
          new_details.update(@finding)
          
          new_details.update(details)
          new_details[:tstamp] = now
          
          id_string = ''
          id_string << request.site
          id_string << request.path
          id_string << new_details[:test_item] if new_details[:test_item]
          id_string << new_details[:class] if new_details[:class]  
          id_string << new_details[:title]  if new_details[:title]
          
          
          if id_string == '' then
            id_string = rand(10000)
          end
          #
          new_details[:fid] = Digest::MD5.hexdigest(id_string)
          
          new_details[:module] = self.class.to_s
          
          new_details[:chat_id] = new_details[:chat].id
          new_details.delete(:chat)
          
          new_finding = Watobo::Core::Finding.new(request, response, new_details)
          #  puts new_finding
          notify(:new_finding, new_finding)
        }
      end
      
      def reset()
        # should be overridden, if counters or status information are used!!!
      end
      
      def resetCounters()
        @numChecks = 0
        @counters = Hash.new
        @progress = 0
        reset()
      end
      
      
      def updateCounters(chat, *prefs)
        @settings[:excluded_parms] = prefs[:excluded_parms] if prefs.is_a? Hash and prefs[:excluded_parms]
        @counters[chat.id] = getCheckCount(chat)
        @numChecks += @counters[chat.id]
        #puts "#{chat.id} : #{@numChecks}"
      end
      
      def urlParmNames(chat)
        begin
          pnames = chat.request.get_parm_names
          @settings[:excluded_parms].each do |p|
            pnames.delete(p)
          end
        rescue => bang
          puts "! no project settings available??"
          puts @project.settings.to_yaml
          puts bang
          puts bang.backtrace
        end
        return pnames
      end
      
      def postParmNames(chat)
        pnames = chat.request.post_parm_names
        begin
          @settings[:excluded_parms].each do |p|
            pnames.delete(p)
          end
        rescue => bang
          puts "! no project settings available??"
          puts @project.settings.to_yaml
          puts bang
          puts bang.backtrace
        end
        return pnames
      end
      
      def getCheckCount(chat)
        count = 0
        generateChecks(chat) do |check|
          count += 1
        end
        return count       
      end
      
      def maxChecks=(m)
        @@max_checks = m
      end
      
      def maxChecks()
        @@max_checks
      end
      
      
      def enabled?
        @enabled
      end
      
      def enabled=(status)
        @enabled = status.is_a? TrueClass|FalseClass
      end
      
      def enable
        @enabled = true
      end
      
      def checksRunning?
        begin
          puts @inner_pool.size
          return true if @inner_pool.size > 0
          return false
        rescue => bang
          p bang
          p bang.backtrace
        end
      end
      
      def disable
        @enable = false
      end
      
      def generateChecks(chat)
        raise "Missing method generateChecks()!!!"
      end
      
      
      def waitLogin(state)
        @@login_in_progress = state
           @inner_pool_cv.signal if state == false
      end
      
      def continue()
        @@pool.each do |thr|
          #  puts "Stopping #{thr}"
          begin
            thr.run if not thr.run?
          rescue
            puts "could not continue thread #{thr}"            
          end
        end
      end
      
      def cancel()
        @@status = :stopped
        @inner_pool.each do |thr|
          #  puts "Stopping #{thr}"
          begin
            Thread.kill( thr ) #.kill if not thr.kill?
            @inner_pool.delete(thr)
          rescue => bang
            puts "could not kill thread #{thr}"
            puts bang
            puts bang.backtrace
          end
        end
        @inner_pool_cv.signal
        
      end
      
      def stop()
        # TODO: real stop/pause function
        cancel()
      end
      
      def fileExists?(request, prefs)
        begin
          t_request, t_response = doRequest(request, prefs)
          
          return false, t_request, t_response if t_response.status =~ /^404/
          return true, t_request, t_response if t_response.status =~ /^403/
          
          if t_request.path_ext != ""
            #TODO: Check for custom error pages
          end
          
          return true, t_request, t_response
        rescue => bang
        end
        return false, nil, nil
      end
      
      
      # +++ run_checks  +++
      # + function: wrapper function for doRequest(r). Needed for additional checks like smartchecks.
      #
      # :run_passive_checks false, 
      # :do_login
      # :pre_chats => []     list of chats to be requested before -> updating CSRF-Tokens
      def run_checks(chat, opts={})
        begin
          reset() # reset variables first
          @@status = :running
          check_opts = { :run_passive_checks => false}
          check_opts.update opts
          @settings.update opts
          
          updateSessionSettings(opts)
          #  puts @session.to_yaml
          
          @@proxy = opts[:proxy] if opts[:proxy]
          
          
          do_test(chat) { |request, response|
            
            begin
              notify(:check_finished, self, request, response)
              if request and response then
                if check_opts[:run_passive_checks] then
                  
                  nc = Watobo::Core::Chat.new(request, response, :id => 0)
                  @project.runPassiveChecks(nc)
                  
                end
                
              end
            rescue => bang
              puts bang
              puts bang.backtrace
            end
          }
          
        rescue => bang
          puts bang
          puts bang.backtrace
          
        end
      end
      
      def do_test(chat, &block)
        # puts chat.request.site
        generateChecks(chat) do |check|
          begin
            @@pool_mutex.synchronize do
              while  @@pool.size >  @@max_checks or @@login_in_progress
                #  puts ".. waiting .. #{@@pool.size}/#{@@max_checks}"
                @@pool_cv.wait(@@pool_mutex)
              end
            end
            
            if @@status != :running
              # puts "..skip.."
              next
            end
            # puts "...create new check..."
            # inner_pool[chat.id] = []             
            Thread.new(check) { |c|
              begin
                @@pool_mutex.synchronize do                  
                  @@pool << Thread.current
                end
                
                while @@login_in_progress == true
                @@login_mutex = Mutex.new
              #  p "#{self.class} waiting for login"
                @@login_cv.wait(@@login_mutex)
                end
                
                #@running_chats << chat
                @inner_pool_mutex.synchronize do
                  @inner_pool << Thread.current unless @inner_pool.include?(Thread.current)
                  
                end
                #  print "_"
                request, response = c.call
                #  print "#"
                retry if @@login_in_progress == true
                
                yield request, response if block
                
              rescue => bang
               # puts "!!!ERROR: running check in #{self.class}"
               # puts bang
               # puts bang.backtrace
                # raise
              ensure
                
                @@pool_mutex.synchronize do
                  
                  @@pool.delete(Thread.current)
                  @@pool_cv.signal
                end
                @inner_pool_mutex.synchronize do
                  @inner_pool.delete(Thread.current)
                  #p "* inner_pool size #{@inner_pool.size} for #{self.class}"
                  @inner_pool_cv.signal
                  
                  #   @running_chats.delete(chat)
                end
              end
            }
          rescue => bang
            puts "RESCUE do_test"
            puts bang
            puts bang.backtrace
          end
        end
        
        @inner_pool_mutex.synchronize do 
          while @inner_pool.size > 0 do 
            #    p "... wait inner_pool #{@inner_pool.size}[#{chat.id}] - #{self.class}" if @inner_pool.size == 0 or @inner_pool.size == 1
            @inner_pool_cv.wait(@inner_pool_mutex) 
            #  sleep(1)
          end
          #p "... leave inner_pool #{@inner_pool.size}[#{chat.id}] - #{self.class}" if @inner_pool.size == 0 or @inner_pool.size == 1
        end
        
        
        
      end
      
      
      
      def initialize(session_name=nil, prefs={})
        #@project = project
        super(session_name, prefs)
        @enabled = true
        # @status = "ready"
        @counters = Hash.new
        @settings = Hash.new
        
        
        @running_chats = []
        
        @numChecks = 0
        @progress = 0
        @check_threads = []
        
        @inner_pool = []
        @inner_pool_mutex = Mutex.new
        @inner_pool_cv = ConditionVariable.new
        
        @checks_cv = ConditionVariable.new
        @checks_mutex = Mutex.new
        
        @info = {
          :check_name => '',    # name of check which briefly describes functionality, will be used for tree and progress views
          :check_group => 'Misc',   # groupname of check, will be used to group checks, e.g. :Generic, SAP, :Enumeration 
          :description => '',   # description of checkfunction
          :author => "not modified", # author of check
          :version => "unversioned",   # check version
          :target => nil               # reserved
          
        }
        
        @finding = {
          :title => 'untitled',          # [String] title name, used for finding tree
          :check_pattern => nil,         # [String] regex of vulnerability check if possible, will be used for highlighting
          :proof_pattern => nil,         # [String] regex of finding proof if possible, will be used for highlighting
          :threat => '',        # threat of vulnerability, e.g. loss of information
          :measure => '',       # measure 
          :class => "undefined",# [String] vulnerability class, e.g. Stored XSS, SQL-Injection, ...
          :subclass => nil,     # reserved
          :type => FINDING_TYPE_UNDEFINED,         # FINDING_TYPE_HINT, FINDING_TYPE_INFO, FINDING_TYPE_VULN 
          :chat => nil,         # related chat must be linked
          :rating=> VULN_RATING_UNDEFINED,  # 
          :cvss => "n/a",       # CVSS Base Vector
          :icon => nil,     # Icon Type
          :timestamp => nil         # timestamp 
        }
        
        @settings[:excluded_parms] = []
        
      end
    end
  end
end
