# .
# scanner.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Core
    
    class Scanner2
      
      attr :numTotalChecks
      attr :numChecksPerModule
      attr :progress
      
      def subscribe(event, &callback)
       (@event_dispatcher_listeners[event] ||= []) << callback
      end
      
      def clearEvents(event)
        @event_dispatcher_listener[event].clear
      end
      
      def notify(event, *args)
        if @event_dispatcher_listeners[event]
          @event_dispatcher_listeners[event].each do |m|
            m.call(*args) if m.respond_to? :call
          end
        end
      end
      
      def running?()
        return true if @status == :running
        return false
      end
      
      def stop(reason="undef")
        begin
          notify(:scanner_stopped, reason)
          @status = :stopped
          puts "* stopping #{@check_list.length} active checks"
          @check_list.each do |mod|
            mod.cancel() 
          end
        rescue => bang
          puts bang
          puts bang.backtrace
        end
      end
      
      def cancel(reason="undef")
        begin
          notify(:scanner_canceled, reason)
          puts "* cancel active checks" unless @check_list.empty?
          @status = :canceled
          @check_list.each do |mod|
            mod.cancel() 
          end
        rescue => bang
          puts bang
          puts bang.backtrace
        end
      end
      
      def siteAlive?(chat)
        site = chat.request.site
        
        return @sites_online[site] if @sites_online.has_key?(site)
        
        
        
        if @prefs[:proxy] =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
          puts "* testing proxy: #{@prefs[:proxy]}"
          # print "* using forwarding proxy #{@project.settings[:proxy]}\r\n"
          use_proxy = true
          host = $1
          port = $2
        else
          puts "* port probing site #{site}"
          host = chat.request.host
          port = chat.request.port
        end
        
        
        begin
          tcp_socket = nil
          #  timeout(6) do
          
          tcp_socket = TCPSocket.new( host, port)
          tcp_socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          tcp_socket.sync = true
          
          if chat.request.proto =~ /https/i then
            @ctx = OpenSSL::SSL::SSLContext.new()
            @ctx.key = nil
            @ctx.cert = nil
            @ctx.tmp_dh_callback = proc { |*args|
              OpenSSL::PKey::DH.new(
                                    OpenSSL::PKey::DH.new(128)
              )
            }
            
            socket = OpenSSL::SSL::SSLSocket.new(tcp_socket, @ctx)
            
            socket.connect
            socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          else
            socket = tcp_socket
          end
          
          if socket.class.to_s =~ /SSLSocket/
            socket.io.shutdown(2)
          else
            socket.shutdown(2)
          end
          socket.close
          puts "* site #{site} is online."
          @sites_online[site] = true
          return true
        rescue Errno::ECONNREFUSED
          p "* connection refused (#{host}:#{port})"
        rescue Errno::ECONNRESET
          puts "* connection reset"
        rescue Errno::EHOSTUNREACH
          p "* host unreachable (#{host}:#{port})"
          
        rescue Timeout::Error
          p "* TimeOut (#{host}:#{port})\n"
          
        rescue Errno::ETIMEDOUT
          p "* TimeOut (#{host}:#{port})"
          
        rescue Errno::ENOTCONN
          puts "!!!ENOTCONN"
        rescue OpenSSL::SSL::SSLError
          p "* ssl error"
          socket = nil
          #  puts "!!! SSL-Error"
          print "E"
        rescue => bang
          puts bang
          puts bang.backtrace
        end
        puts "!!! site #{site} is offline."
        @sites_online[site] = false
        return false
        #        if @sites_online.has_key?(site) then
        #          return @sites_online[site] ? true : false
        #        end
        
        #        if @onlineCheck.isOnline?(chat) then
        #          puts "Site #{site} is online"
        #          @sites_online[site] = true
        #          return true
        #        else
        #    puts "Site #{site} is offline"
        #          @sites_online[site] = false
        #          return false
        #        end
      end
      
      def continue()
        puts "!!! Scan Running !!!"
        @active_checks.each do |mod|
          mod.run()
        end
        @status_running = true
      end
      
      def updateProgress(test_module, progress_step)
        @progress += progress_step
        @progress_display.progress = @progress if @progress_display
      end
      
      
      def run( check_prefs={} ) 
        @status_running = true
        @sites_online.clear
        @uniqueRequests = Hash.new
        @status = :running
        @check_list = []
        @login_count = 0
        @max_login_count = 20        
        
        # counter for fired checks
        checks = 0
        
        m = "Starting Scan ..."
        notify(:logger, LOG_INFO, m)
        puts m
        #scan_session = Time.now.to_i
        @filtered_chat_list.each do |chat|
          if @prefs[:online_check] == false or siteAlive?(chat) then
            @active_checks.each do |mod|
              print "*"
              # accept Class- and Check-Types
              check = mod
              # puts @prefs[:scan_session]
              check = mod.new(@prefs[:scan_session], @prefs ) if mod.respond_to? :new
              @check_list.push check
              
              check.subscribe(:logout) { |m|
                # do_login = false
                #  @login_mutex.synchronize do
                next if @login_count > @max_login_count or @prefs[:auto_login] == false
                if @login_mutex.try_lock                 
                  begin                       
                    m.waitLogin(true)
                    p
                    puts "!LOGOUT DETECTED!"
                    puts "* try login ..."
                    
                    @login_count += 1
                    m.runLogin(@prefs[:login_chats])
                    
                    m.waitLogin(false) if m
                  rescue => bang
                    puts "! Could not relogin"
                    puts bang
                    puts bang.backtrace
                  ensure
                  end
                  puts "... done!"
                  @login_mutex.unlock
                end
                
              }
              
              check.subscribe(:check_finished) do |m, request, response|
                notify(:progress, m)
                next if @prefs[:scanlog_dir] == ''
                next unless File.exist?(@prefs[:scanlog_dir]) or File.ftype(@prefs[:scanlog_dir]) == "directory"
                Thread.new(request, response) {|req, resp|
                  begin
                    fname = File.join(@prefs[:scanlog_dir], "chat_" + Time.now.to_f.to_s)
                    retry if File.exist?(fname)
                    settings = {
                    :request => req,
                    :response => resp
                    }
                    File.open(fname, "w") { |fh|
                      YAML.dump(settings, fh)
                    }
                    
                  
                rescue => bang
                  puts bang
                  puts bang.backtrace
                end
                }
              end
              
              check.subscribe(:new_finding) do |f|
                #    p "* NEW FINDING"
                #   p f.details[:module]
                notify(:new_finding, f)
              end
              
              notify(:logger, LOG_INFO, "running checks: #{check.info[:check_name]} for chat #{chat.id}")
              checks += 1
              Thread.new(check, chat, check_prefs ){|m, c, p|
                @scan_pool_mutex.synchronize do
                  @scan_pool << Thread.current
                end
                begin  
                  m.run_checks(c,p) 
                  
                rescue => r
                  puts "!!!ERROR!!!"
                  #  puts "!!!Starting scan for module #{mod.info[:check_name]}"
                  #print "\nstopped working on chat(#{c.id}): #{r}\n"
                  puts r.backtrace
                ensure
                  @scan_pool_mutex.synchronize do
                    @scan_pool.delete(Thread.current)
                    @scan_pool_cv.signal
                  end
                  # p "* Check #{m.class} finished"
                  # notify(:module_finished, m)
                  notify(:logger, LOG_INFO, "finished checks: #{check.info[:check_name]} for chat #{chat.id}")
                  
                end              
              }
              
            end
          end
        end
        #scan_running = true
        puts
        puts "[Scanner] All #{checks} checks fired and running ..."
        
        @scan_pool_mutex.synchronize do
          while @scan_pool.size > 0 do
            print "\r..#{@scan_pool.size} checks running     "
            @scan_pool_cv.wait(@scan_pool_mutex) 
          end
        end
        print "\r..#{@scan_pool.size} checks running     \n"
        puts "* Scan Finished *"
        
        return 0
      end
      
      def initialize(chat_list=[], active_checks=[], passive_checks=[], prefs={})
        # @project = project        # needed for centralized session management
        
        
        @numTotalChecks = 0
        @numChecksPerModule = Hash.new
        @chat_list = chat_list
        @active_checks = active_checks
        @passive_checks = passive_checks
        
        @login_in_progress = false
        @login_mutex = Mutex.new
        @login_cv = ConditionVariable.new
        
        @filtered_chats = []
        
        #  @thread_list = Hash.new
        @scan_pool = []
        @scan_pool_mutex = Mutex.new
        @scan_pool_cv = ConditionVariable.new
        
        @check_list = []
        
        @sites_online = Hash.new
        @event_dispatcher_listeners = Hash.new
        @status = :stopped
        # @onlineCheck = OnlineCheck.new(@project)
        
        notify(:logger, LOG_INFO, "Initializing Scanner ...")
        
        @prefs = {
          #:root_path => [],
          #:excluded_chats => [],
          :smart_scan => true,
          :excluded_parms => [],
          #:non_uniq_parms => [],
          :login_chat_ids => [],
          :auto_login => true,
          # :valid_sids => Hash.new,
          :sid_patterns => [],
          :run_passive_checks => false,
          :proxy => '',
          :scanlog_dir => '',
          :scan_session => Digest::MD5.hexdigest(Time.now.to_f.to_s),
          :check_online => true
        }
        
        
        @prefs.update prefs 
        #puts "set up scanner"
        #puts @prefs[:login_chats]
        #puts @prefs[:logout_signatures]
        # puts @prefs.to_yaml
        
        @filtered_chat_list = filteredChats(@chat_list, @prefs)
        
        @active_checks.uniq.each do |m|
          check = m
          check = m.new(:dummy) if check.respond_to? :new
          check.resetCounters()
          # puts "* updating counters for module #{m.info[:check_name]}"
          #  puts "* updating #{@chat_list.length} chats"
          @filtered_chat_list.each do |chat|
            
            check.updateCounters(chat, @prefs)
          end
          
          @numTotalChecks += check.numChecks
          cn = check.info[:check_name]
          # puts "+ add check: #{cn}"
          notify(:logger, LOG_INFO, "add check #{cn}")
          @numChecksPerModule[cn] = check.numChecks
        end
        
        notify(:logger, LOG_INFO, "Scanner Ready!")  
      end
      
      private
      
      # Function: filteredChats
      # It's primarily used to reduce the number of requests for the scan.
      # Returns [] of chats which fulfill the requirements. 
      # Inputs:
      # - chats: list of chats[]
      # - prefs: :non_unique_parms=>[], :smart_scan=>true|false
      def filteredChats(chats, prefs = {})
        check_ids = Hash.new
        fchats = []
        hstring = ''
        if prefs[:smart_scan] == true then
          chats.each do |chat|
            ps = chat.request.parm_names.sort
            if prefs[:non_unique_parms] and prefs[:non_unique_parms].length > 0 then
              ps.map!{|p| 
                if prefs[:non_unique_parms].include?(p) then
                  p += chat.request.get_parm_value(p)
                end                
              }
            end
            
            next if chat.request.site.nil?
            hstring = chat.request.site + chat.request.path + chat.request.file + ps.join 
            rhash = Digest::MD5.hexdigest(hstring)
            
            next if check_ids.has_key?(rhash)
            # p hstring
            check_ids[rhash] = nil
            fchats.push chat
          end
        else
          fchats = chats
        end
        return fchats
      end
    end
    
  end
end