# .
# session.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'socket'
require 'timeout'
require 'openssl'
require 'optparse'
require 'digest/md5'
require 'lib/httpparser.rb'
require 'lib/qShapers.rb'
require 'lib/http_socket.rb'


module Watobo
  module Core
    
    class Session
      
      @@settings = Hash.new
      @@proxy = ''
      
      @@lock = Mutex.new
      
      @@login_mutex = Mutex.new
      @@login_cv = ConditionVariable.new
      
      def subscribe(event, &callback)
       (@event_dispatcher_listeners[event] ||= []) << callback
      end
      
      def clearEvents(event)
        @event_dispatcher_listener[event].clear
      end
      
      def notify(event, *args)
        if @event_dispatcher_listeners[event]
          @event_dispatcher_listeners[event].each do |m|
            m.call(*args) if m.respond_to? :call
          end
        end
      end
      
      def runLogin(chat_list, prefs={})
        login_prefs = Hash.new
        login_prefs.update prefs
        dummy = {:ignore_logout => true, :update_sids => true, :update_session => true, :update_contentlength => true}
        login_prefs.update dummy
       # puts "* RUN LOGIN"
        #  puts login_prefs.to_yaml
        chat_list.each do |chat|
          test_req = chat.copyRequest
          request, response = doRequest(test_req, login_prefs)
        end
        @@login_cv.signal
        # exit
        #  print "L]"
      end
      
      #     def sessionSettings=(prefs)
      #       applySessionSettings(prefs)
      #     end
      
      def sessionSettings()
        @@settings
      end
      #
      # getHTTPHeader
      #    
      def getHTTPHeader(raw_request, prefs)
        begin
          #  print(".")    
          socket, sent_request = sendHTTPRequest(raw_request, prefs)
          if socket.nil? then
            #   print("x")
            return nil, raw_request, sent_request
          end
          header = readHTTPHeader(socket)
          #print("o")
        rescue => bang
          puts bang
        end
        
        return socket, sent_request, header
      end
      
      # +++ doRequest(request)  +++
      # + function: 
      #
      def doRequest(request, opts={})
        begin
          socket, request = sendHTTPRequest(request, opts)
          
          if socket.nil?
            return nil, request
          end
          response = readHTTPHeader(socket)
          
          update_sids(request, response.headers) if opts[:update_sids] == true
          
          readHTTPBody(socket, response, request, opts)
          
          unless response.body.nil?
            update_sids(request, response.body) if opts[:update_sids] == true and response.content_type =~ /text\//
          end
          
          socket.close
          
        rescue  => bang
          puts "! Error in doRequest"
          puts "Exception!! in Module #{Module.nesting[0].name}"
          puts bang
           puts bang.backtrace
          @lasterror = bang
          # raise
          # ensure
        end
        
        response.extend Watobo::Parser::Web10
        return request, response
      end
      
      def proxy=(p)
        @@proxy = p
      end
      
      def proxy()
        @@proxy
      end
      
      #
      # INITIALIZE
      #
      # Possible preferences:
      # :proxy => '127.0.0.1:port'
      # :valid_sids => Hash.new,
      # :sid_patterns => [],
      # :logout_signatures => [],
      # :update_valid_sids => false, 
      # :update_sids => false, 
      # :update_contentlength => true
      def initialize(session_id, prefs={})
        @event_dispatcher_listeners = Hash.new
        @session = nil
        
        session_id = Digest::MD5.hexdigest(Time.now.to_f.to_s) if session_id.nil?
        
        if !@@settings[session_id] then
          @@settings[session_id] = {
            :valid_sids => Hash.new,
            :sid_patterns => [],
            :logout_signatures => [],
            :logout_content_types => Hash.new,
            :update_valid_sids => false, 
            :update_sids => false, 
            :update_session => true,
            :update_contentlength => true,
            :login_chats => []
          } 
        end
        
        @session = @@settings[session_id] # shortcut to settings
        @session.update prefs
        
        
        @@proxy = prefs[:proxy] if prefs.is_a? Hash and prefs[:proxy]
        
        @socket = nil
        
        
        # TODO: Implement switches for URL-Encoding (http://www.blooberry.com/indexdot/html/topics/urlencoding.htm)
        # TODO: Implement switches for Following Redirects
        # TODO: Implement switches for Logging, Debugging, ...
      end
      
      
      def readHTTPBody(socket, response, request, prefs={})
        clen = response.content_length
        data = ""
        #   timeout(5) do
        begin
          if  clen > 0           
            Watobo::Framework::HTTP.read_body(socket) do |c|
              data += c
              break if data.length == clen
            end          
          else          
            #   puts "* no content-length information ... mmmmmpf"
            eofcount = 0
            Watobo::Framework::HTTP.read_body(socket) do |c|
              data += c
            end
            
          end
        rescue => e
          puts "!!! ERROR: read_response"
          puts e
        end
        # end
        
        response.push data
        unless prefs[:ignore_logout]==true  or @session[:logout_signatures].empty?
          notify(:logout, self) if loggedOut?(response)         
        end
        update_sids(request, response) if prefs[:update_sids] == true
        
      end
      
      private
      def loggedOut?(response, prefs={})
        begin
          return false if @session[:logout_signatures].empty?
          response.each do |line|
            @session[:logout_signatures].each do |p|
              #     puts "!!!*LOGOUT*!!!" if line =~ /#{p}/
              return true if line =~ /#{p}/
            end
          end
        rescue => bang
          puts bang
          puts bang.backtrace
        end
        return false
      end
      
      def readHTTPHeader(socket, prefs={})
        header = []
        begin
          Watobo::Framework::HTTP.read_header(socket) do |line|
            # puts line.unpack("H*")
            header.push line
          end
        rescue => bang
          puts "!ERROR: read_header"
          return nil
        end
        header.extend Watobo::Parser::Url
        header.extend Watobo::Parser::Web10
        #  update_sids(header)
        
        update_sids(request, response) if prefs[:update_sids] == true
        
        unless prefs[:ignore_logout]==true or @session[:logout_signatures].empty?
          notify(:logout, self) if loggedOut?(header)
        end
        
        return header
      end
      
      
      
      #     def read_response(socket)
      
      
      #       return response
      #    end
      
      def update_sids(request, response)
        #   p "* update sids"
        Thread.new{
          begin
            site = request.site
            @@lock.synchronize do 
              response.each do |line|
                # puts line 
                @session[:sid_patterns].each do |pat|   
                  if line =~ /#{pat}/i then
                    sid_key = Regexp.quote($1.upcase)
                    sid_value = Regexp.quote($2)
                    #print "U"
                 #   puts "GOT NEW SID (#{sid_key}): #{sid_value}"
                    @session[:valid_sids][site] = Hash.new if @session[:valid_sids][site].nil? 
                    @session[:valid_sids][site][sid_key] = sid_value
                  end
                end
                
              end
            end
          rescue => bang
            puts bang
            puts bang.backtrace
            
          end
        }
      end
      
      def updateSessionSettings(settings={})
        [:valid_sids, :sid_patterns, :logout_signatures, :logout_content_types, :update_valid_sids, :update_sids, :update_session, :update_contentlength, :login_chats].each do |k|
          @session[k] = settings[k] if settings[k]
        end
      end
      
      def updateSession(request)
        
        if @session[:valid_sids].has_key?(request.site)
          # puts "* found sid for site: #{request.site}"
          request.map!{ |line|
            res = line
            @session[:sid_patterns].each do |pat|
              begin                
                if line =~ /#{pat}/i then  
                  sid_key = Regexp.quote($1.upcase)
                  old_value = Regexp.quote($2)                  
                  if @session[:valid_sids][request.site].has_key?(sid_key) then
                    if not old_value =~ /#{@session[:valid_sids][request.site][sid_key]}/ then # sid value has changed and needs update
                #      print "S"
                  #    puts "+ update sid #{sid_key}"
                  #    puts "-OLD: #{old_value}"
                  #    puts "-NEW: #{@session[:valid_sids][request.site][sid_key]}"
                      
                #      puts "---"
                      # dummy = Regexp.quote(old_value)
                      res = line.gsub!(/#{old_value}/, @session[:valid_sids][request.site][sid_key])
                      if not res then puts "!!!could not update sid (#{sid_key})"; end
                      #     puts "->#{line}"
                    end
                  end
                end
              rescue => bang
                puts bang
                puts bang.backtrace
                # puts @session.to_yaml
              end
            end
            res
          }
        end
        
      end
      
      def sendHTTPRequest(request, prefs={})
        
        begin
          @lasterror = nil
          #response = []
          use_proxy = false
          if @@proxy =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
            # print "* using forwarding proxy #{@project.settings[:proxy]}\r\n"
            use_proxy = true
            host = $1
            port = $2
          else
            host = request.host
            port = request.port
          end
          # update current preferences, prefs given here are stronger then global settings!
          current_prefs = Hash.new       
          [:update_session, :update_sids, :update_contentlength, :ssl_cipher].each do |k|
         
            current_prefs[k] = prefs[k].nil? ? @session[k] : prefs[k]
          end
          
          updateSession(request) if current_prefs[:update_session] == true
          
          #---------------------------------------
          request.removeHeader("^Proxy-Connection") #if not use_proxy
          request.removeHeader("^Connection") #if not use_proxy
          request.removeHeader("^Accept-Encoding")
          # If-Modified-Since: Tue, 28 Oct 2008 11:06:43 GMT
          # If-None-Match: W/"3975-1225192003000"
          request.removeHeader("^If-")
          #  puts
          #  request.each do |line|
          #  puts line.unpack("H*")
          #end
          #puts
          if current_prefs[:update_contentlength] == true then
            request.fix_content_length()
          end 
          
          # puts "========== Add Headers"
          request.addHeader("Connection", "Close") #if not use_proxy
          request.addHeader("Proxy-Connection", "Close") #if not use_proxy
          request.addHeader("Accept-Encoding", "None") #don't want encoding
          #request.add_header("Via", "Watobo") if use_proxy
          #puts request
          # puts "=============="
        rescue => bang
          puts bang
          puts bang.backtrace
        end
        begin
          tcp_socket = nil
          #  timeout(6) do
          
          tcp_socket = TCPSocket.new( host, port)
          tcp_socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          tcp_socket.sync = true
          #  end
          
          if request.proto =~ /https/i then
            # NEED SPECIAL HANDSHAKE IF SSL-PROXY IS IN PLACE
            if @@proxy =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
              tcp_socket.print "CONNECT #{request.host}:#{request.port} HTTP/1.0\r\n"+
              # "Proxy-Connection: Keep-Alive\r\n" +
                                 "\r\n"
              while line = tcp_socket.gets   
                break if line.length < 3
              end  
              
            end
            #@ctx = OpenSSL::SSL::SSLContext.new('SSLv23_client')
            @ctx = OpenSSL::SSL::SSLContext.new()
            @ctx.key = nil
            @ctx.cert = nil
            
            @ctx.ciphers = current_prefs[:ssl_cipher] if current_prefs[:ssl_cipher]
            
            @ctx.tmp_dh_callback = proc { |*args|
              OpenSSL::PKey::DH.new(
                                    OpenSSL::PKey::DH.new(128)
              )
            }
            
            socket = OpenSSL::SSL::SSLSocket.new(tcp_socket, @ctx)
            
            socket.connect
            socket.setsockopt(Socket::SOL_SOCKET, Socket::SO_KEEPALIVE, 1)
          else
            socket = tcp_socket
          end
          
          # remove URI before sending request
          uri_cache = nil
          # if not @project.settings[:proxy] =~ /(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):(\d{1,4})/ then
          uri_cache = request.removeURI if use_proxy == false              
          # end
          
          # puts request          
          data = request.join + "\r\n" 
          
          socket.print data
          
          # RESTORE URI FOR HISTORY/LOG
          request.restoreURI(uri_cache) if use_proxy == false
          #  puts request.first
          
        rescue Errno::ECONNREFUSED
          # response.push "WATOBO: connection refused (#{host}:#{port})"
          print "C!"
          request = "WATOBO: connection refused (#{host}:#{port})"
          socket = nil
        rescue Errno::ECONNRESET
          puts '!!! connection reset'
        rescue Errno::EHOSTUNREACH
          request = "WATOBO: host unreachable (#{host}:#{port})"
          socket = nil
        rescue Timeout::Error
          request = "WATOBO: TimeOut (#{host}:#{port})\n"
          socket = nil
        rescue Errno::ETIMEDOUT
          #response.push 
          #response.push request.first if request
          request = "WATOBO: TimeOut (#{host}:#{port})"
          socket = nil
          #rescue EOFError
          #puts "EOF"
          #printf("O\r\n")
        rescue Errno::ENOTCONN
          puts "!!!ENOTCONN"
        rescue OpenSSL::SSL::SSLError
          request = nil
          socket = nil
          #  puts "!!! SSL-Error"
          # print "E"
        rescue => bang
          #   puts "!!! Uuuuppppsssss ...."
          #   puts request.to_yaml
          #   puts bang
          
          #   puts bang.backtrace
        end
        
        # @project.extendRequest(request) if request
        
        return socket, request
      end
      
      private
      
      def applySessionSettings(prefs)        
        [ :update_valid_sids, :update_session, :update_contentlength, :valid_sids, :sid_patterns, :logout_signatures ].each do |v|       
          @@settings[v] = prefs[v] if prefs[v]
        end
      end
      
    end
  end
end