# .
# check_policy_menu.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/gui/checkboxtree.rb'

module Watobo
  module Gui
    class ChecksPolicyFrame < FXVerticalFrame
      attr :policy_name
      
      def onPolicyChanged(sender, sel, item)
        @policy_name = @policyCombo.getItemText(@policyCombo.currentItem)
        policy = @policyCombo.getItemData(@policyCombo.currentItem)
        applyPolicy(policy)
      end
      
      def isElementChecked(data)
        item = @tree.findItemByData(data)
        item.checked
      end
      
      def applyPolicy(policy)
        
        tree_elements = []
        @checks.each do |check|
          status = true
          begin
            status = policy[check.class.to_s]
          rescue
            puts "unknown policy or unknown test [#{@policy}] - [#{check.class.to_s}"
          end
          a = {
            :name => "#{check.info[:check_group]}|#{check.info[:check_name]}",
            :enabled => status,
            :data => check
          }
          tree_elements.push a
        end
        
        showTree(tree_elements)
      end
      
      def showInfo(check)
        
      end
      
      def showTree(elements)
        @tree.createTree(elements)
      end
      
      def getSelectedModules()
        sel = @tree.getCheckedData
        #sel.map { |i| p i.class }
      end
      
      def initialize(parent, checks, policy)
        # Invoke base class initialize function first        
        super(parent, :opts => LAYOUT_FILL_X| LAYOUT_FILL_Y, :padding => 0)
        
        @policy_name = nil  # name of active policy
        @policy = nil         # policy settings
        @checks = checks
        
        
        policy_frame = FXHorizontalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
                
        @policy_name = policy[:name]
        @policy_list = policy[:list]
        @policy = @policy_list[@policy_name]
        
        policy_count = ( @policy_list.is_a? Hash ) ? @policy_list.length : 0
       
        @policyCombo = FXComboBox.new(policy_frame, policy_count, nil, 0,
                                      COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FILL_X)
        
        @policyCombo.numVisible = policy_count
        @policyCombo.connect(SEL_COMMAND, method(:onPolicyChanged))
        
        tree_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        @tree = CheckBoxTreeList.new(tree_frame)
        if @policy_list
        @policy_list.each do |pname, p|
          
          @policyCombo.appendItem(pname, p)
        end
        # select policy
        index = @policyCombo.findItem(@policy_name)
        if index >= 0 then
          @policyCombo.setCurrentItem(index)
        end
        applyPolicy(@policy)
        end
      end
    end
    
   
    #--
  end
end