# .
# conversation_table.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    TABLE_COL_METHOD = 0x0001
    TABLE_COL_HOST = 0x0002
    TABLE_COL_PATH = 0x0004
    TABLE_COL_PARMS = 0x0008
    TABLE_COL_STATUS = 0x0010
    TABLE_COL_COOKIE = 0x0020
    TABLE_COL_COMMENT = 0x0040
    
    class ConversationTable < FXTable
      attr_accessor :autoscroll
      
      def showConversation(chat_list)
        clearConversation()
        chat_list.each do |chat|
          addChat(chat)
        end
      end
      
      def clearConversation()
        #   begin
        #     @col_order.each do |col|
        #       pos = @col_order.index(col)
        #       @cell_width[col] = self.getColumnWidth(pos) 
        #         end
        #   rescue => bang
        #     puts bang
        #   end
        
        self.clearItems        
        initColumns()        
        adjustCellWidth()
      end
      
      def adjustCellWidth()
        begin
          self.rowHeader.width = 50
          @cell_width.each do |col, width|
            pos = @col_order.index(col)
            self.setColumnWidth(pos, width)
          end
        rescue => bang
          puts "!!!ERROR: adjustCellWidth"
        end
        
      end
      
      def addChat(chat, *prefs)
        
        if self.getNumRows <= 0 then
          clearConversation()
          # initColumns() 
        end
        
        lastRowIndex = self.getNumRows
        self.appendRows(1)
        
        self.rowHeader.setItemJustify(lastRowIndex, FXTableItem::LEFT)        
        self.setRowText(lastRowIndex, chat.id.to_s)
        
        index = @col_order.index(TABLE_COL_METHOD)
        
        self.setItemText(lastRowIndex, index, chat.request.method)
        self.getItem(lastRowIndex, index).justify = FXTableItem::LEFT
        
        index = @col_order.index(TABLE_COL_HOST)
        self.setItemText(lastRowIndex, index, chat.request.host)
        self.getItem(lastRowIndex,index).justify = FXTableItem::LEFT
        
        index = @col_order.index(TABLE_COL_PATH)
        self.setItemText(lastRowIndex, index, chat.request.path)
        self.getItem(lastRowIndex, index).justify = FXTableItem::LEFT
        
        index = @col_order.index(TABLE_COL_PARMS)
        ps = ""
        if chat.request.method =~ /POST/ then
        ps = chat.request.post_parms.join("&")  
        else
          ps = chat.request.urlparms        
        end
        parms = ""
         unless ps.nil?
          parms = ps[0..50]
          parms += "..." if ps.length > 50
          
           parms.gsub!(/[^[:print:]]/,'.')
           end
          self.setItemText(lastRowIndex, index, parms)
        self.getItem(lastRowIndex, index).justify = FXTableItem::LEFT
        
        index = @col_order.index(TABLE_COL_STATUS)
        self.setItemText(lastRowIndex, index, chat.response.status)
        self.getItem(lastRowIndex,index).justify = FXTableItem::LEFT
        
        if chat.response.header_value("set-cookie").first then
          index = @col_order.index(TABLE_COL_COOKIE)
          self.setItemText(lastRowIndex, index, chat.response.header_value("set-cookie").first.chomp) 
          self.getItem(lastRowIndex, index).justify = FXTableItem::LEFT
        end
        
        if chat.comment then
          index = @col_order.index(TABLE_COL_COMMENT)
          self.setItemText(lastRowIndex, index, chat.comment) 
          self.getItem(lastRowIndex, index).justify = FXTableItem::LEFT
        end
        
        self.makePositionVisible(self.numRows-1, 0) if @autoscroll == true
      end
      
      def initColumns() 
        self.setTableSize(0, @columns.length)
        self.visibleRows = 20
        self.visibleColumns = @columns.length
        
        @columns.each do |type, name|
          index = @col_order.index(type)
          self.setColumnText( index, name ) 
          # puts self.getItem(@col_order.index(col), 0  ).class.to_s
        end
        
      end
      
      
      def initialize(owner, prefs)
        super(owner, :opts => TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY|LAYOUT_SIDE_TOP, :padding => 2)
        
        #   FXMAPFUNC(SEL_CLICKED, FXTable::ID_SELECT_CELL, :onSelectCell)
        
        @columns = Hash.new
        @cell_width = Hash.new
        @col_order = []
        @autoscroll = false
        
        @columns = Hash.new
        
        @columns[TABLE_COL_METHOD] = "Method" 
        @columns[TABLE_COL_HOST] = "Host" 
        @columns[TABLE_COL_PATH] = "Path" 
        @columns[TABLE_COL_PARMS] = "Parameters" 
        @columns[TABLE_COL_STATUS] = "Status" 
        @columns[TABLE_COL_COOKIE] = "Set-Cookie"
        @columns[TABLE_COL_COMMENT] = "Comment" 
        
        # initialize columns order  
        @col_order = [ TABLE_COL_METHOD, TABLE_COL_HOST, TABLE_COL_PATH, TABLE_COL_PARMS, TABLE_COL_STATUS, TABLE_COL_COOKIE, TABLE_COL_COMMENT ]
        
        # init cell width
        @cell_width = Hash.new
        @cell_width[TABLE_COL_METHOD] = 50 
        @cell_width[TABLE_COL_HOST] = 120 
        @cell_width[TABLE_COL_PATH] = 200 
        @cell_width[TABLE_COL_PARMS] = 150 
        @cell_width[TABLE_COL_STATUS] = 50 
        @cell_width[TABLE_COL_COOKIE] = 70
        @cell_width[TABLE_COL_COMMENT] = 100 
        
        initColumns()
        
        self.columnHeader.connect(SEL_CHANGED) do |sender, sel, index|
          type = @col_order[index]
          @cell_width[type] = self.getColumnWidth(index) 
        end
        
        self.columnHeader.connect(SEL_COMMAND) do |sender, sel, index|
          type = @col_order[index]
          if self.getColumnWidth(index) == @cell_width[type]  then
            self.fitColumnsToContents(index)           
            if self.getColumnWidth(index) < @cell_width[type]  then # all cells are empty -> resize to original width              
              self.setColumnWidth(index, @cell_width[type])
            end
          else
            self.setColumnWidth(index, @cell_width[type])
          end
         
        end
        
        adjustCellWidth()
        self.setBackColor(FXRGB(255, 255, 255))
        self.setCellColor(0, 0, FXRGB(255, 255, 255))
        self.setCellColor(0, 1, FXRGB(255, 240, 240))
        self.setCellColor(1, 0, FXRGB(240, 255, 240))
        self.setCellColor(1, 1, FXRGB(240, 240, 255))
        
      end
      
    end
  end
end