# .
# findings_tree.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    class FindingsTree < FXTreeList
      attr_accessor :project
      
      include Watobo::Core
      
      def subscribe(event, &callback)
       (@event_dispatcher_listeners[event] ||= []) << callback
      end
      
      
      
      def refresh(project)
        
      end
      
      def expandFullTree(item)
        self.expandTree(item)
        item.each do |c|
          expandFullTree(c) if !self.itemLeaf?(c)
        end
      end
      
      def collapseFullTree(item)
        self.collapseTree(item)
        item.each do |c|
          collapseFullTree(c) if !self.itemLeaf?(c)
        end
      end
      
      def hidden?(finding)
        #if @project then
        #   @interface.default_settings[:domain_filters].each do |domainfilter|
        #  @project.settings[:domain_filters].each do |domainfilter|
        # puts domainfilter
        #    return true if finding.request.site =~ /#{domainfilter}/ 
        #@filtered_domains[site] = domainfilter
        # end
        #  end
        #end
        false
      end
      
      def reload()
        self.clearItems
        
        @findings.each_value do |finding|
          
          addFinding(finding)
          
        end
        #        @findings_tree.expandTree(project_leaf)
        # @interface.updateRequestTable(@project)
      end
      
      
      
      
      def hideDomain(domain_filter)
        #@interface.default_settings[:domain_filters].push domain_filter
        #@interface.updateTreeLists
        #notify(:new_domain_filter, domain_filter)  
      end
      
      
      def addFinding(finding)
        #  p "* add finding to tree"
      #  puts finding.details[:title]
        @findings[finding.details[:fid]] = finding
        if @show_scope_only == true
          addFindingItem(finding) if @project.siteInScope?(finding.request.site)
        else
          addFindingItem(finding)
        end
        
      end
      
      def addFindingItem(finding)
        begin
          
          site = nil
          # puts "add finding"
          if not hidden?(finding) then
            site = self.findItem(finding.request.site, nil,SEARCH_FORWARD|SEARCH_IGNORECASE)
            
            if not site then
              # found new site
              site = self.appendItem(nil, finding.request.site, ICON_PROJECT, ICON_PROJECT)
              self.appendItem(site, "Vulnerabilities", ICON_VULN, ICON_VULN)
              self.appendItem(site, "Hints", ICON_HINTS, ICON_HINTS)
              self.appendItem(site, "Info", ICON_INFO, ICON_INFO)
              #site = @findings_tree.moveItem(project.first,project,site)
              self.setItemData(site, :item_type_site)
              
            end
            
            
            finding_type=""
            
            case finding.details[:type]
              when FINDING_TYPE_INFO
              finding_type = "Info"
              icon = ICON_INFO_INFO
              
              when FINDING_TYPE_HINT
              finding_type = "Hints"
              icon = ICON_HINTS_INFO
              
              
              when FINDING_TYPE_VULN
              finding_type = "Vulnerabilities"
              if finding.details[:rating] == VULN_RATING_LOW
                icon = ICON_VULN_LOW
                #  puts "low-rating-vuln"
              end
              if finding.details[:rating] == VULN_RATING_MEDIUM
                icon = ICON_VULN_MEDIUM
              end
              if finding.details[:rating] == VULN_RATING_HIGH
                icon = ICON_VULN_HIGH
              end
              if finding.details[:rating] == VULN_RATING_CRITICAL
                icon = ICON_VULN_CRITICAL
              end
            end
            
            
            
            sub_tree = self.findItem(finding_type, site, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
            if sub_tree and sub_tree.parent == site and finding.details[:class]
              
              class_item = self.findItem(finding.details[:class], sub_tree, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not class_item or class_item.parent != sub_tree
                class_item = self.appendItem(sub_tree, finding.details[:class], icon, icon)
              end
              title_item = self.findItem(finding.details[:title], class_item, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not title_item or title_item.parent != class_item
                title_item = self.appendItem(class_item, finding.details[:title], nil, nil)
                self.setItemData(title_item, :title )
                # puts finding.details[:title]
              end
              #   puts title_item
              resource = finding.request.path_ext
              
              request_item = self.findItem(resource, title_item, SEARCH_FORWARD|SEARCH_IGNORECASE|SEARCH_NOWRAP)
              if not request_item or request_item.parent != title_item
                text = "/" + resource
                request_item = self.appendItem(title_item, text)
                self.setItemData(request_item, finding)
              end
            end
            
          end
        rescue => bang
          puts "!ERROR: could not add finding to tree"
          puts bang
          puts bang.backtrace
          
        end
      end
      
      
      
      
      def initialize(parent, interface, project)
        @project = project
        @interface = interface
        @parent = parent
        @findings = Hash.new
        @show_scope_only = false
        
        @event_dispatcher_listeners = Hash.new
        
        super(parent, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_RIGHT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|TREELIST_ROOT_BOXES|TREELIST_EXTENDEDSELECT)
        
        @filtered_domains = Hash.new # domains which already have been filtered
        
        self.connect(SEL_COMMAND) do |sender, sel, item|
          if self.itemLeaf?(item)
            getApp().beginWaitCursor do
              begin
                if item.data
                  if item.data.is_a? Watobo::Core::Finding
                    #@interface.show_vuln(item.data)
                    notify(:vuln_click, item.data)                  
                  end
                end
              rescue => bang
                puts "!!! Error: could not show selected finding"
                puts bang
              end
            end
          elsif item.data == :title then
            #@interface.show_vuln(item.first.data) if item.first.data
            notify(:vuln_click, (item.first.data)) if item.first.data          
          end
        end
        
        self.connect(SEL_DOUBLECLICKED) do |sender, sel, item|
          if self.itemLeaf?(item)           
            begin
              puts item.data.class
              if item.data.is_a? Watobo::Core::Finding
                #TODO: show vulnerability details
                # @interface.showFindingInfo(item.data)
                notify(:finding_click, item.data)
              else
                puts item.data.class.to_s
              end
            rescue => bang
              puts "!!! Error: could not show selected finding"
              puts bang
            end
          end          
        end
        
        self.connect(SEL_RIGHTBUTTONRELEASE) do |sender, sel, event|          
          unless event.moved?
            FXMenuPane.new(self) do |menu_pane|
            item = sender.getItemAt(event.win_x, event.win_y)
             target = FXMenuCheck.new(menu_pane, "show scope only" )
                
                target.check = @show_scope_only
                
                
                target.connect(SEL_COMMAND) { |sender, sel, item|
                  @show_scope_only = sender.checked?
                  reload() if @project
                }
                
            unless item.nil?           
              
                data = item ? self.getItemData(item) : nil 
                
                FXMenuSeparator.new(menu_pane)
                if !self.itemLeaf?(item)
                  FXMenuCommand.new(menu_pane, "expand tree" ).connect(SEL_COMMAND) {
                    expandFullTree(item)
                  }
                  
                  FXMenuCommand.new(menu_pane, "collapse tree" ).connect(SEL_COMMAND) {
                    self.collapseFullTree(item)
                  }                      
                end
                
                if data == :item_type_site then
                  FXMenuSeparator.new(menu_pane)
                  FXMenuCommand.new(menu_pane, "add site to scope" ).connect(SEL_COMMAND) {
                    
                    notify(:add_site_to_scope, item.to_s)
                  }
                  
                elsif data.is_a? Watobo::Core::Finding then
                  FXMenuSeparator.new(menu_pane)
                  doManual = FXMenuCommand.new(menu_pane, "Manual Request.." )
                  doManual.connect(SEL_COMMAND) {
                    # @interface.open_manual_request_toolkit(item.data)
                    notify(:open_manual_request, item.data)
                    
                  }
                  info = FXMenuCommand.new(menu_pane, "Details..." )
                  info.connect(SEL_COMMAND) {
                    #@interface.showFindingDetails(item.data)}
                    notify(:show_finding_details, item.data)
                  }
                end
                
                
                
              end
              menu_pane.create
                menu_pane.popup(nil, event.root_x, event.root_y)
                app.runModalWhileShown(menu_pane)
            end
          end
        end
      end
      
      private
      
      def notify(event, *args)
        if @event_dispatcher_listeners[event]
          @event_dispatcher_listeners[event].each do |m|
            m.call(*args) if m.respond_to? :call
          end
        end
      end
    end
    # namespace end
  end
end