# .
# progress_window.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    class ProgressWindow < FXTopWindow
      
      def newProject_UNUSED(project, settings)
        @project = project
        Thread.new(project, settings) { |p, s|
          
          p = Project.new(@gui, settings)
          p.subscribe(:update_progress){ |prefs|
            if prefs[:progress]
              @pbar.increment(prefs[:progress]) 
              @step.text = "#{@pbar.progress}/#{@pbar.total}"
            end
            @pbar.total += prefs[:total] if prefs[:total]
            @title.text = prefs[:title] if prefs[:title]
            @job.text = prefs[:job] if prefs[:job]
         
          }
          p.setupProject()
          
          @gui.setActiveProject(p)
          begin
            @job.text = "Finishing ..."
            @step.text = "Starting Proxy"
            
            cert_path = File.join(@gui.watobo_base, "certificates")
            proxy = InterceptProxy.new(@gui.active_project, cert_path)
            proxy.run
            @gui.iproxy = proxy
          rescue => bang
            puts bang
          ensure
            self.hide
          end
        }
      end
      
      def updateProgress(prefs)
         begin
            if prefs[:progress]
              @pbar.increment(prefs[:progress]) 
              @step.text = "#{@pbar.progress}/#{@pbar.total}"
            end
            @pbar.total += prefs[:total] if prefs[:total]
            @title.text = prefs[:title] if prefs[:title]
            @job.text = prefs[:job] if prefs[:job]
            @pbar.progress = 0 if prefs[:finished]
            puts "+ finished" if prefs[:finished]
            self.update
          rescue => bang
            puts bang
            puts bang.backtrace
            end
      
      end
      
      def register(object)
        @rt = Thread.new(object) { |o|
        
          o.subscribe(:update_progress){ |prefs|
          begin
            if prefs[:progress]
              @pbar.increment(prefs[:progress]) 
              @step.text = "#{@pbar.progress}/#{@pbar.total}"
            end
            @pbar.total += prefs[:total] if prefs[:total]
            @title.text = prefs[:title] if prefs[:title]
            @job.text = prefs[:job] if prefs[:job]
            @pbar.progress = 0 if prefs[:finished]
            puts "+ finished" if prefs[:finished]
            self.update
          rescue => bang
            puts bang
            puts bang.backtrace
            end
          }
        }
      end
      
      def finish()
        @rt.kill if @rt.alive?
      end
      
      def initialize(owner, title, total)
        @gui = owner
        
        super( owner, title, nil, nil, DECOR_BORDER, 0, 0, 250, 100, 0, 0, 0, 0, 0, 0)
        
        title_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        prog_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        job_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        step_frame = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_SIDE_TOP)
        
        @title = FXLabel.new(title_frame, title )
        @job = FXLabel.new(job_frame, 'JOB' )
        @step = FXLabel.new(step_frame, 'current step' )
        
        @pbar = FXProgressBar.new(prog_frame, nil, 0, LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_HORIZONTAL)
        
        
        @pbar.progress = 0
        @pbar.total = total
        @pbar.barColor=0
        @pbar.barColor=FXRGB(255,0,0)
        
      end
    end
  end
end
