# .
# proxy_dialog.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
module Watobo
  module Gui
    class AddProxyDialog < FXDialogBox
      attr :proxy_name
      attr :proxy
      attr :proxy_port
      
      include Responder
      
      def onAccept(sender, sel, event)
        if @proxy_name.value != "" then
          if  @proxy.value =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ then
            if   @proxy_port.value =~ /^\d{1,5}$/ then
              getApp().stopModal(self, 1)
              self.hide()
              return 1
            else
              FXMessageBox.information(self, MBOX_OK, "Wrong Format", "Port format is wrong (e.g. 8080")
            end
            
          else
            FXMessageBox.information(self, MBOX_OK, "Wrong Format", "Proxy (IP only) format is wrong (e.g. 1.2.3.4")
          end
        else
          FXMessageBox.information(self, MBOX_OK, "Proxy Name Missing", "You need to set a name for the new proxy (e.g. myproxy)")
        end
        
      end
      
      def initialize(owner)
        
        super(owner, "Add Proxy", :opts => DECOR_TITLE|DECOR_BORDER)
        
        FXMAPFUNC(SEL_COMMAND, ID_ACCEPT, :onAccept)
        
        @proxy_name = FXDataTarget.new('')
        @proxy_port = FXDataTarget.new('')
        @proxy = FXDataTarget.new('')
        
        main = FXVerticalFrame.new(self, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        
        nameFrame = FXHorizontalFrame.new(main, :opts => LAYOUT_FILL_X)          
        FXLabel.new(nameFrame, "Proxy Name:", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        FXTextField.new(nameFrame, 25,
                        :target => @proxy_name, :selector => FXDataTarget::ID_VALUE,
                        :opts => TEXTFIELD_NORMAL|LAYOUT_SIDE_RIGHT)
        
        proxyFrame = FXHorizontalFrame.new(main, :opts => LAYOUT_FILL_X)
        FXLabel.new(proxyFrame, "Proxy (IP only):", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        FXTextField.new(proxyFrame, 25,
                        :target => @proxy, :selector => FXDataTarget::ID_VALUE,
                        :opts => TEXTFIELD_NORMAL|LAYOUT_SIDE_RIGHT)
        
        
        portFrame = FXHorizontalFrame.new(main, :opts => LAYOUT_FILL_X)
        FXLabel.new(portFrame, "Port:", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        FXTextField.new(portFrame, 25,
                        :target => @proxy_port, :selector => FXDataTarget::ID_VALUE,
                        :opts => TEXTFIELD_NORMAL|LAYOUT_SIDE_RIGHT)
        
        
        buttons = FXHorizontalFrame.new(main, :opts => LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
                                        :padLeft => 40, :padRight => 40, :padTop => 20, :padBottom => 20)
        
        accept = FXButton.new(buttons, "&Accept", nil, self, ID_ACCEPT,
        FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        accept.enable
        # Cancel
        FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
        FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
      end
    end
    
    
    class ProxyDialog < FXDialogBox
      attr :proxy
      
      #TODO: Implement Connection Check!!
      
      attr_accessor :proxy
      
      
      include Responder
      
      def switchEnableProxy()
        if not @proxyEnableCheckButton.checked? then 
          @proxyList.hide()
          @newProxyButton.disable
          @delProxyButton.disable
          @acceptBtn.enable
          @proxy = ''
          @proxyList.killSelection()
        else
          @proxyList.show()
          @newProxyButton.enable
          selectProxy(@proxy)
          @acceptBtn.disable
          refreshProxyList()
        end
        @proxySelectionFrame.enabled = @proxyEnableCheckButton.checked?
        @proxy_prelabel.enabled = @proxyEnableCheckButton.checked?
        @proxy_label.enabled = @proxyEnableCheckButton.checked?
      end
      
      def refreshProxyList
        @proxyList.clearItems(false)
        @proxyList.setTableSize(0, 2)
        return if @proxy_list.nil?
        @proxy_list.each do |name, proxy|
          addProxyItem(name, proxy)
        end
      end
      
      def getProxyPrefs()
        proxy_prefs = Hash.new
        proxy_prefs = {
        :list => @proxy_list,
        :name => @proxy
        }
        return proxy_prefs
      end
      
      def onAccept(sender, sel, event)
          # @settings[:proxies] = proxies
          getApp().stopModal(self, 1)
          self.hide()
          return 1
    
      end
      
      def onCancel(sender, sel, event)
        getApp().stopModal(self, 0)
        self.hide()
        return 1
      end
      
      def addProxy(sender, sel, event)
        pdlg = AddProxyDialog.new(self)
        
        if pdlg.execute != 0
          last = @proxyList.getNumRows
          @proxy = pdlg.proxy_name.value
          new_proxy = {
            :proxy => pdlg.proxy.value,
            :port => pdlg.proxy_port.value,
            :description => ""
          }
          @proxy_list[@proxy] = new_proxy
          
          addProxyItem(@proxy, new_proxy)
          @proxyList.selectRow(last)
          @proxyList.makePositionVisible(last,0)
          @proxy_label.text = @proxy
          @acceptBtn.enable
          @acceptBtn.setFocus()
          @acceptBtn.setDefault()
        end
      end
      
      def delProxy(sender, sel, event)
        
        row = @proxyList.getCurrentRow
      #  puts row
        if row >= 0 and @proxyList.rowSelected?(row) then
          p = @proxyList.getItem(row,1).to_s
          #puts @proxy.class.to_s
       #   puts "delete: #{proxy}"
          @proxy = ''
          @proxyList.removeRows(row,1)
          @proxyList.killSelection(true)
          @delProxyButton.disable
          @acceptBtn.disable
          @proxy_label.text = ''
        end
      end
      
      def onProxySelect(sender, sel, index)
        
        row = sender.getCurrentRow
        if row >= 0 then
          @proxyList.selectRow(row)
          @proxy = @proxyList.getItem(row,0).to_s
          @proxy_label.text = @proxy
          @delProxyButton.enable
          @acceptBtn.enable
          @acceptBtn.setFocus()
          @acceptBtn.setDefault()
        end
      end
      
      def selectProxy(proxy_name)
        @proxyList.killSelection(true)
        @proxy_label.text = "- none -"
        #   @proxy = ''
       # puts proxy_name
        i = 0
        @proxyList.each do |items|
          if items[0].to_s == proxy_name
            @proxyList.selectRow(i)
            @proxyList.makePositionVisible(i,0)
            @proxy = @proxyList.getItem(i,0).to_s
            @proxy_label.text = @proxy
            @delProxyButton.enable
          end
          i += 1
        end
      end
      
      def addProxyItem(name, proxy)
        lastRowIndex = @proxyList.numRows        
        @proxyList.appendRows(1)
        @proxyList.rowHeader.setItemJustify(lastRowIndex,FXTableItem::LEFT)
        @proxyList.setItemText(lastRowIndex, 0, name)
        @proxyList.getItem(lastRowIndex,0).justify = FXTableItem::LEFT
        @proxyList.setItemText(lastRowIndex, 1, proxy[:proxy] + ":" + proxy[:port])
        @proxyList.getItem(lastRowIndex,1).justify = FXTableItem::LEFT
      end
      
      def initialize(owner, proxy_settings)
        super(owner, "Use Proxy", :opts => DECOR_TITLE|DECOR_BORDER, :width => 250, :height => 400)
        @new_proxy = FXDataTarget.new('')
        @proxy_settings = Hash.new
        @proxy_settings.update(proxy_settings)
        
        @proxy = @proxy_settings[:name]
        @proxy_list = @proxy_settings[:list]
       
        
        
        FXMAPFUNC(SEL_COMMAND, ID_ACCEPT, :onAccept)
        FXMAPFUNC(SEL_COMMAND, ID_CANCEL, :onCancel)
        
        
        current_proxy_frame = FXHorizontalFrame.new(self,FRAME_NONE|LAYOUT_FILL_X|FRAME_GROOVE)
        FXLabel.new(current_proxy_frame, "Current Proxy:", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        @cp_label = FXLabel.new(current_proxy_frame, "- none -", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        
        main_frame = FXVerticalFrame.new(self, :opts => FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        enable_proxy_frame = FXHorizontalFrame.new(main_frame,FRAME_NONE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
        @proxyEnableCheckButton = FXCheckButton.new(enable_proxy_frame, "enable forwarding proxy", nil, 0, ICON_BEFORE_TEXT|LAYOUT_SIDE_LEFT)
        
        @proxyEnableCheckButton.connect(SEL_COMMAND) do |sender, sel, item|
          switchEnableProxy()          
        end 
        
        @proxySelectionFrame = FXGroupBox.new(main_frame, "Select Forwarding Proxy", LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 0, 0)
        
        proxySubSelection = FXVerticalFrame.new(@proxySelectionFrame, :opts => LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_GROOVE)
        
        @proxyList = FXTable.new(proxySubSelection,
                                 :opts => TABLE_COL_SIZABLE|TABLE_ROW_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y|TABLE_READONLY,
                                 :padding => 2)
        @proxyList.setTableSize(0, 2)
        
        @proxyList.connect(SEL_COMMAND, method(:onProxySelect))
        @proxyList.connect(SEL_SELECTED, method(:onProxySelect))
        
        @proxyList.rowHeaderWidth = 0
        @proxyList.rowHeaderMode = LAYOUT_FIX_WIDTH
        
        @proxyList.visibleRows = 5
        @proxyList.setColumnText(0, "Name" )
        @proxyList.setColumnText(1, "Proxy" )
        
        
        
        @newProxyButton = FXButton.new(proxySubSelection, "Add" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        @newProxyButton.connect(SEL_COMMAND, method(:addProxy))
        
        @delProxyButton = FXButton.new(proxySubSelection, "Delete" ,  nil, nil, :opts => BUTTON_NORMAL|LAYOUT_RIGHT)  
        @delProxyButton.connect(SEL_COMMAND, method(:delProxy))
        
        
        selection_frame = FXHorizontalFrame.new(main_frame,FRAME_NONE|LAYOUT_FILL_X)
        @proxy_prelabel = FXLabel.new(selection_frame, "Selected Proxy:", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        @proxy_label = FXLabel.new(selection_frame, "- none -", nil, LAYOUT_TOP|JUSTIFY_RIGHT)
        
        buttons = FXHorizontalFrame.new(main_frame, :opts => LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
                                        :padLeft => 40, :padRight => 40, :padTop => 20, :padBottom => 20)
        
        # Accept
        @acceptBtn = FXButton.new(buttons, "&Accept", nil, self, ID_ACCEPT,
        FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        @acceptBtn.disable
        # Cancel
        FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
        FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
        
        
        
        @proxyEnableCheckButton.setCheck(false, false)
        @proxyEnableCheckButton.setCheck(true, false) if @proxy != '' 
        refreshProxyList()
        switchEnableProxy()
        
        if @proxy != ''
          @cp_label.text = @proxy
          selectProxy(@proxy)
        end
        
      end
    end
    
  end
end