# .
# httpparser.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/core/watobo_constants'

# http://www.ietf.org/rfc/rfc2396.txt
# http://en.wikipedia.org/wiki/URI_scheme

#
# http://www.mysite.com:80/my/path/show.php?p=aaa&debug=true
#
# proto = "http"
# host = "www.mysite.com"
# site = "www.mysite.com:80"
# dir = "my/path"
# file = "show.php"
# file_ext = "show.php?p=aaa&debug=true"
# path = "my/path/show.php"
# query = "p=aaa&debug=true"
# fext = "php"
# path_ext = "my/path/show.php?p=aaa&debug=true"

module Watobo
  module Parser
    
    module Url
      
      def file
        if self.first =~ /\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}[^\?]*\/([^\?]*) HTTP.*/
          return $1
        else
          return ""
        end   
      end
      
      def file_ext
        if self.first =~ /\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}[^\?]*\/(.*) HTTP.*/
          return $1
        else
          return ""
        end   
      end
      
      def urlparms
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        if parts[1] then
          if parts[1] =~ /(.*) HTTP/i then          
            return $1
          else
            return nil
          end
        else
          return nil
        end 
      end
      
      # returns the resource string
      # e.g. URI = http://www.mysite.com/path/resource.php?var=1&t=true
      # returns "path/resource.php?var=1&t=true"
      # No trailing slash!
      
      # def resource
      #   dummy = self.first.gsub(/\?+/,"?")
      #   parts = dummy.split('?')
      #   parts[0].gsub!(/ HTTP\/(.*)/i,"")
      
      #   if parts[0] =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)(.*)/i then          
      #     return $2
      #   else
      #     return ""
      #   end        
      # end
      
      def method
        if self.first =~ /(\w+) http/i then          
          return $1
        else
          return nil
        end
      end
      
      
      #The path may consist of a sequence of path segments separated by a
      #single slash "/" character.  Within a path segment, the characters
      #"/", ";", "=", and "?" are reserved.  Each path segment may include a
      #sequence of parameters, indicated by the semicolon ";" character.
      #The parameters are not significant to the parsing of relative
      #references.
      
      #
      # http://www.mysite.com:80/my/path/show.php?p=aaa&debug=true
      # path = "my/path/show.php"
      
      def __path
        begin
          dummy = self.first.gsub(/\?+/,"?")
          parts = dummy.split('?')
          
          parts[0].gsub!(/ HTTP\/(.*)/i,"") if parts[0]
          if parts[0] and parts[0] =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)(.*\/)/i then          
            return $2
          else
            return ""
          end
        rescue => bang
          puts "!!! HTTP-Parser (path): #{bang}"
        end
      end
      
      def path
        if self.first =~ /^\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/([^\?]*).* HTTP/i then
          return $1
        else
          return ""
        end
      end
      
      # path_ext = "my/path/show.php?p=aaa&debug=true"
      def path_ext
        if self.first =~ /^\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/(.*) HTTP\//i then
          return $1
        else
          return ""
        end
      end
      
      def dir
        if self.first =~ /^\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/([^\?]*)\/.* HTTP/i then
          return $1
        else
          return ""
        end
      end
      
      
      # returns resource and query of an uri
      # e.g. http://www.mysite.com/path/resource.php?var=1&t=true
      # returns [ "/path/resource.php", "var=1&t=true" ]
      # No trailing slash!
      
      def __query
        begin
          resource = nil
          q = nil
          if self.first =~ /\w+ (.*) HTTP.*/ then
            uri = $1
          end
          parts = uri.split('?')
          resource = parts.shift
          resource.gsub!(/^https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}/, '')
          resource.gsub!(/.*\//, '')          
          if parts.length > 0
            q = parts.join('?').gsub(/ HTTP.*$/,'')
          end
        rescue => bang
          puts "!!! Could not parse query !!!"
          puts bang
          puts bang.backtrace
        end
        return resource, q
        
      end     
      
      def query
        begin
          q = nil
          if self.first =~ /\w+ (.*) HTTP.*/ then
            uri = $1
          end
          parts = uri.split('?')
          parts.shift
          puts "HTTPParser.query: #{parts.join('?')}"
          return parts.join('?')
        rescue => bang
          puts "!!! Could not parse query !!!"
          puts bang
          puts bang.backtrace
        end
        return nil
        
      end     
      
      
      def element
        cl = self.first.gsub(/\?+/,"?")
        cl.gsub!(/ HTTP.*/, '')
        dummy = cl.split('?').first       
        if dummy =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}).*\/(.*)/i then          
          return $2
        else
          return ""
        end        
      end
      
      def doctype
        dummy = self.first.gsub(/\?+/,"?")
        parts = dummy.split('?')
        parts[0].gsub!(/ HTTP\/(.*)/i,"")
        if parts[0] =~ /(.*\.)(\w{2,3})$/i then          
          return $2
        else
          return ''
        end
      end
      
      def proto
        dummy = self.first
        result = "http"
        #  puts dummy
        if dummy =~ /\w+ https:\/\//i then          
          result = "https"
        end
        return result
      end
      
      def url        
        #dummy = self.first.dup        
        #dummy.gsub!(/ HTTP\/(.*)/i,"")
        #dummy.gsub!(/^\w+ /,"")      
        #return dummy.strip
        if self.first =~ /^\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}.*) HTTP\//i then
          return $1
        else
          return nil
        end        
      end
      
      
      def site
        #dummy = self.first.gsub(/\?+/,"?")
        #parts = dummy.split('?')
        #if parts[0] =~ /\w+ https?:\/\/([\-0-9a-zA-Z.]*[:0-9]{0,6})/i then          
        if self.first =~ /^\w+ https?:\/\/([\-0-9a-zA-Z.]*[:0-9]{0,6})/i then
          return $1
        else
          return nil
        end
      end
      
    
      
      def host
        #  dummy = self.first
        #  parts = dummy.split('?')
        #  if parts[0] =~ /^\w+ https?:\/\/([\-0-9a-zA-Z.]*)[:0-9]{0,6}/i then          
        if self.first =~ /^\w+ https?:\/\/([\-0-9a-zA-Z.]*)[:0-9]{0,6}/i then
          return $1
        else
          return nil
        end
      end
      
      # returns all subdir combinations
      # www.company.com/this/is/my/path.php
      # returns:
      # [ "/this", "/this/is", "/this/is/my" ]
      def subDirs
        sub_dirs = self.dir.split(/\//)
        dir = ""
        sub_dirs.map! do |d| dir += "/" + d ; end
        return sub_dirs
      end
      
      
      
      def port
        dummy = self.first
        portnum = nil
        parts = dummy.split('?')
        
        if parts[0] =~ /\w+ https:\/\//i then
          portnum = 443
        elsif parts[0] =~ /\w+ http:\/\//i
          portnum = 80
        end
        if parts[0] =~ /\w+ https?:\/\/[\-0-9a-zA-Z.]*:([0-9]{0,6})/i then          
          portnum = $1
        end
        return portnum
      end
      
      def get_parms
        parmlist=[]
        if self.first =~ /\w+ (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}).*\/.*(\?.*=.*) HTTP/i then
          dummy = $2.gsub!(/\?+/,"?").split('?')
          # remove left part of ? from url 
          dummy.shift
          
          parmlist=dummy.join.split(/\&/)
        end
        return parmlist
        
      end
      
      #################### doubles
      
      def get_parm_names
        
        parm_names=[]
        parmlist=[]
        parmlist.concat(get_parms)
        
        parmlist.each do |p| 
          if p then
            p.gsub!(/=.*/,'')
            parm_names.push p
          end
        end
        
        return parm_names
        
      end
      
      def get_parm_value(parm_name)
        parm_value = ""
        self.get_parms.each do |parm|
          if parm =~ /^#{Regexp.quote(parm_name)}=/i then
            dummy = parm.split(/=/)
            if dummy.length > 1 then
              #  parm_value=dummy[1].gsub(/^[ ]*/,"")  
              parm_value=dummy[1].strip
            end
          end
        end
        return parm_value
      end
      
      def post_parm_value(parm_name)
        parm_value=""
        self.post_parms.each do |parm|
          if parm =~ /#{Regexp.quote(parm_name)}/i then
            dummy = parm.split(/=/)
            if dummy.length > 1 then
              parm_value = dummy[1].strip
            else
              # puts "Uhhhh ... need parameter value from '#{parm}''"
            end
          end
        end
        return parm_value
      end
      
      
      
    end
    
    module Web10
      include Watobo::Core
      
      
      def post_parms
        parmlist=[]
        if self.last =~ /\=.*\&?/i
          parmlist = self.last.split(/\&/)        
        end
        return parmlist
      end
      
      def parms
        parmlist=[]
        parmlist.concat(get_parms)
        parmlist.concat(post_parms)
        
        return parmlist
      end
      
      def parm_names
        
        parm_names=[]
        parmlist=[]
        parmlist.concat(get_parms)
        parmlist.concat(post_parms)
        parmlist.each do |p| 
          p.gsub!(/=.*/,'')
          parm_names.push p
        end
        
        return parm_names
        
      end
      
      def post_parm_names
        
        parm_names=[]
        parmlist=[]
        
        parmlist.concat(post_parms)
        parmlist.each do |p|
          if p then
            p.gsub!(/=.*/,'') 
            parm_names.push p
          end
        end
        
        return parm_names
        
      end
      
 #     def get_parm_names
        
 #       parm_names=[]
 #       parmlist=[]
 #       parmlist.concat(get_parms)
        
 #       parmlist.each do |p| 
  #        if p then
 #           p.gsub!(/=.*/,'')
 #           parm_names.push p
 #         end
 #       end
        
 #       return parm_names
        
 #     end
      
       def header_value(header_name)
        header_values =[]
        self.headers.each do |header|
          if header =~ /^#{header_name}/i then
            dummy = header.split(/:/)
            value=dummy[1]
            value.gsub!(/^[ ]*/,"")
            header_values.push value
          end
        end
        return header_values
      end
      
      def content_type
        ct = "undefined"
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Content-Type: (.*)/i then 
            ct = $1
            break
          end
        end
        return ct.strip
      end
      
      def content_length
        # Note: Calculate Chunk-Encoded Content-Length
        # this is only possible if the whole body is loaded???
        ct = -1
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Content-Length: (.*)/i then 
            ct = $1.to_i
            break
          end
        end
        return ct
      end
      
      def transferEncoding
        te = TE_NONE
        self.each do |line|
          break if line.strip.empty?
          if line =~ /^Transfer-Encoding: (.*)/i then
            dummy = $1.strip
            puts dummy
            te = case dummy 
              when 'chunked'
              Watobo::Core::TE_CHUNKED
              when 'compress'
              Watobo::Core::TE_COMPRESS
              when 'zip'
              Watobo::Core::TE_GZIP
              when 'deflate'
              Watobo::Core::TE_DEFLATE
              when 'identity'
              Watobo::Core::TE_IDENTITY
            end
            break
          end
        end
        return te
      end
      
      
      def contentMD5
        hash = Digest::MD5.hexdigest(self.body)
        return hash
      end
      
      
#      def get_parm_value(parm_name)
#        parm_value = ""
#        self.get_parms.each do |parm|
#          if parm =~ /^#{Regexp.quote(parm_name)}=/i then
#            dummy = parm.split(/=/)
#            if dummy.length > 1 then
#              #  parm_value=dummy[1].gsub(/^[ ]*/,"")  
#              parm_value=dummy[1].strip
#            end
#          end
#        end
#        return parm_value
#      end
      
      def post_parm_value(parm_name)
        parm_value=""
        self.post_parms.each do |parm|
          if parm =~ /#{Regexp.quote(parm_name)}/i then
            dummy = parm.split(/=/)
            if dummy.length > 1 then
              parm_value = dummy[1].strip
            else
              # puts "Uhhhh ... need parameter value from '#{parm}''"
            end
          end
        end
        return parm_value
      end
      
      
     def body
      # puts "=BODY"
     #  puts self.last
       return self.last if self[-2].strip.empty?
       return nil
     end
      
      def body2
        
        if self.join =~/[^\r\n\r\n]*\r\n\r\n(.*)/ then 
          result = $1
          return result
        else
          return ''
        end
      end
      
      def status
        dummy = self.first
        
        if dummy =~ /^HTTP\/1.. (.*)/i then          
          return $1.chomp
        else
          return nil
        end        
      end
      
         def headers
        header_list=[]
        self.each do |line|
          return header_list if line.strip.empty?
          header_list.push line
        end
        return header_list
      end
      
      def cookies
        cookie_list=[]
        self.headers.each do |line|
          if line =~ /Cookie2?: (.*)/i then
            clist = $1.split(";")
            clist.each do |c|
              c.gsub!(/^[ ]+/,"")
              c.chomp!
              cookie_list.push c
            end
          end
        end
        return cookie_list
      end
      
      
      def data
        return self.last.strip if self.last =~ /\=.*\&?/i
        return ""
      end
      
     
    end
    
    ############################################################
    # R E S P O N S E   P A R S E R   M I X I N 
    ###############################################################
    module ResponseParser
      def status
        dummy = self.first
        
        if dummy =~ /HTTP\/1.. (.*)/i then          
          return $1.chomp
        else
          return nil
        end        
      end
      
      
      
      def headers
        header=[]
        self.each do |line|
          return header if line.length <3 or line =~ /^\r\n$/
          header.push line
        end
      end
      
      
    end
  end
end
