# .
# qShapers.rb
#   
# Copyright 2010 by siberas, http://www.siberas.de
# 
# This file is part of WATOBO (Web Application Tool Box)
#        http://watobo.sourceforge.com
# 
# WATOBO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2 of the License.
# 
# WATOBO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with WATOBO; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
# .
require 'lib/core/watobo_constants'

module Watobo
  module Framework
    module Shaper
      module Web10
        
        def replace_post_parm(parm,value)
          parm_quoted = Regexp.quote(parm)
          self.last.gsub!(/([?&]{0,1})#{parm_quoted}=([0-9a-zA-Z\-\._,+<>\%!=]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
        end
        
        def replace_get_parm(parm,value)
          parm_quoted = Regexp.quote(parm)
          # puts "replacing parameter #{parm} with value #{value}"
          # self.first.gsub!(/([?&]{0,1})#{parm}=([0-9a-zA-Z\-\._,+<>\%!=]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
          self.first.gsub!(/([?&]{1})#{parm_quoted}=([^ &]*)(&{0,1})/i, "\\1#{parm}=#{value}\\3")
        end
        
        def replaceMethod(new_method)
          self.first.gsub!(/^\w*/i, "#{new_method}")
        end
        
        def replaceFileExt(new_file)
          #   puts "replace element #{new_element}"
          new_file.gsub!(/^\//, "")
          self.first.gsub!(/([^\?]*\/)(.*) (HTTP.*)/i,"\\1#{new_file} \\3")
        end
        
        def replaceElement(new_element)
          #   puts "replace element #{new_element}"
          new_element.gsub!(/^\//, "")
          self.first.gsub!(/(.*\/)(.*) (HTTP.*)/i,"\\1#{new_element} \\3")
        end
        
        def replaceQuery(new_query)
          #   puts "replace element #{new_element}"
          new_element.gsub!(/^\//, "")
          self.first.gsub!(/(.*\/)(.*) (HTTP.*)/i,"\\1#{new_element} \\3")
        end
        
        def strip_path()          
          if self.first =~ /(^\w+ https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}.*\/).* (HTTP.*)/i then          
            new_line = "#{$1} #{$2}\r\n"
            self.shift
            self.unshift(new_line)
          end
          # puts "* StripPath: #{self.first}"
        end
        
        def setDir(dir)
          dir.gsub!(/^\//,"")
          self.first.gsub!(/(.* https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)(.*)( HTTP\/.*)/, "\\1#{dir}\\3")
        end
        
        def appendDir(dir)
          dir.gsub!(/^\//,"")
          self.first.gsub!(/(.* https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}.*\/).*( HTTP\/.*)/, "\\1#{dir}\\2") 
          
        end
        
        def add_post_parm(parm,value)
          line = self.last
          return false if line !~ /=/
          line += "&#{parm}=#{value}" 
          self.pop
          self.push line
        end
        
        def add_get_parm(parm,value)
          line = self.shift
          new_p = "&"
          new_p = "?" if not self.element =~ /\?/
          new_p += parm
          line.gsub!(/( HTTP\/.*)/, "#{new_p}=#{value}\\1")
          self.unshift(line)
        end
        
        def addHeader(header, value)
          self_copy = []
          self_copy.concat(self.headers)
          self_copy.push "#{header}: #{value}\r\n"
          
          body = self.body
          
          if body then
            self_copy.push "\r\n"
            #self_copy.concat(self.body)
            self_copy.push body
          end
          
          self.replace(self_copy)
          
        end
        
        def removeURI
          if self.first =~ /^(\w+) (https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/ then
            uri = $2
            self.first.gsub!(/^(\w+) (#{Regexp.quote(uri)})/,"\\1 /")
            # puts "* Removed URI: #{uri}"
            # puts self.first
            return uri
          else
            return nil
          end
          #self.first.gsub!(/^(.*)(https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/,"\\1/")
        end
        
        def restoreURI(uri)
          if self.first =~ /^(\w+) \/(.*)/ then
            method = $1
            rest = $2
            self.first.gsub!(/^\w.*/, "#{method} #{uri}#{rest}")
            
          else
            return nil
          end
          #self.first.gsub!(/^(.*)(https?:\/\/[\-0-9a-zA-Z.]*[:0-9]{0,6}\/)/,"\\1/")
        end
        
        #
        # R E M O V E _ H E A D E R
        #
        def removeHeader(header)
          self_copy = []
          eoh = false
          self.each do |line|
            # puts "+ + + #{line}"
            if not eoh == true then
              if not line =~ /#{header}/i
                self_copy.push line
              end  
            else
              self_copy.push line
            end
            
            if line.strip.length == 0 then
              #  puts "header finished"
              eoh = true 
            end
          end
          self.replace(self_copy)
        end
        
        def replace_header(header, value)
          
        end
        
        
        def fix_session(pattern,value)
          
        end
        
        def fix_content_length
          self_copy = []
          self.each do |line|
            if line =~ /^Content-Length/i
              #  puts "--- fix content-length ---"
              #  puts line
              #  puts self.body.join.length.to_s
              #  puts "---"
              self_copy.push "Content-Length: #{self.body.length.to_s}\r\n" if self.body
              #self_copy.push "Content-Length: #{self.body.join("\r\n").length.to_s}\r\n"
            else
              self_copy.push line
            end         
          end
          self.replace(self_copy)
        end
        
        
        def fixupContentLength
          te = self.transferEncoding
          if te == Watobo::Core::TE_CHUNKED then
            # puts "Transfer-Encoding = TE_CHUNKED"
            # puts self.body
            self.removeHeader("Transfer-Encoding")
            self.addHeader("Content-Length", "0")
            new = []
            new.concat self.headers
            new.push "\r\n"
            
            
            bytes_to_read = 0
            
            body = []
            
            dummy = self.body.split(/\n/)
            dummy.each{ |e|            
              if bytes_to_read == 0 and e.strip =~ /^([0-9a-fA-F]{1,4})$/ then
                bytes_to_read += $1.hex
                puts bytes_to_read.to_s
                break if bytes_to_read == 0
              else
                body.push "#{e}" 
                #puts e
                bytes_to_read -= (e.length + 1) 
              end
              
              
            }
            
            new.push body.join("\n")
            self.replace(new)
            self.fix_content_length
            # puts "= FIXED ="
            # puts self.headers
          elsif te == Watobo::Core::TE_NONE then
            self.fix_content_length
          end
          
        end
        
        def fixupContentLength_UNUSED
          te = self.transferEncoding
          if te == Watobo::Core::TE_CHUNKED then
            # puts "Transfer-Encoding = TE_CHUNKED"
            # puts self.body
            self.removeHeader("Transfer-Encoding")
            self.addHeader("Content-Length", "0")
            new = []
            new.concat self.headers
            new.push "\r\n"
            
            #dummy = self.body.join.split(/\n/)
            dummy = self.body.split(/\n/)            
            chunk = dummy.first.strip.hex
            dummy.shift
            bytes_to_read = chunk
            line_count = 0
            body = []
            dummy.each{ |e|
              line_count += 1
              if e.strip =~ /^[0-9a-fA-F]{1,4}$/ then
                # don't know exactly if hexvalues are chunk ... mmmhh
                # so the maximum difference should be line_count * 2 (\r\n)
                if bytes_to_read >= line_count*2 then
                  body.push "#{e}\n"
                  bytes_to_read -= e.length
                else
                  bytes_to_read = e.strip.hex
                  line_count = 0
                end
              else
                new.push "#{e}\n" 
                bytes_to_read -= e.length
              end
            }
            
            #          chunk = dummy.shift.hex
            #          puts "Chunk-Size: #{chunk.to_s}"
            #          i = 0
            #          while chunk > 0 
            #          puts "Chunk-Size: #{chunk.to_s}"
            #            c = 0
            #            while c < chunk and not dummy[i].strip =~ /[0..9a..fA..F]*/
            #              c += dummy[i].length
            #              new.push dummy[i]
            #              i += 1
            #            end
            #chunk = dummy[i].hex
            #             puts dummy[i]
            #             puts chunk.to_s
            #             i += 1
            #             puts i.to_s
            #           end
            self.replace(new)
            self.fix_content_length
            # puts "= FIXED ="
            # puts self.headers
          elsif te == Watobo::Core::TE_NONE then
            self.fix_content_length
          end
          
        end
        
        def appendQueryParms(parms)
          return if parms.nil?
          return if parms == ''
          
          puts self.first
          puts self.file_ext
          
          pref = (self.file_ext =~ /\?/) ? '&' : '?'  
          puts "append query parms"
          self.first.gsub!(/(.*) (HTTP\/.*)/, "\\1#{pref}#{parms} \\2")
          
        end
        
        def setData(data)
          return if data.nil?
          if self.last =~ /\=.*\&?/i
            self.pop 
            self.push data if data != ""
          end
        end
        
        def setMethod(method)
          self.first.gsub!(/(^\w+) /, "#{method} ")
        end
        
        def setHTTPVersion(version)
          self.first.gsub!(/HTTP\/(.*)$/, "HTTP\/#{version}")
          puts "HTTPVersion fixed: #{self.first}"
        end
        
      end
      
      
    end
  end
end
